import sys

# If our base template isn't on the PYTHONPATH already, we need to do this:
sys.path.append('../path/to/base/templates')

import basetemplate

class AlteredTemplate(basetemplate.BaseTemplate):
    """This project only needs an S3 bucket, but no EC2 server."""

    def add_resources(self):
        self.add_bucket()

    def add_bucket(self):
        """This will add a bucket using the base template, and then add a custom CORS 
        configuration to it."""

        super(AlteredTemplate, self).add_bucket()
        self.resources['StaticFiles']['Properties']['CorsConfiguration'] = {
            'CorsRules': [
                {
                    'AllowedHeaders': ['*'],
                    'AllowedMethods': ['GET'],
                    'AllowedOrigins': ['*'],
                }
            ]
        }

cft = AlteredTemplate("S3 Bucket Project", options)
cft.add_resources()



# Start with a template...
cft = CloudFormationTemplate(description="A very small template.")

cft.resources.ec2_instance = Resource('AnInstance', 'AWS::EC2::Instance',
    Properties({
        # This is an ubuntu AMI, picked from http://cloud-images.ubuntu.com/
        # You may need to change this if you're not in the us-east-1 region
        # Or if Ubuntu deregisters the AMI
        'ImageId': 'ami-c30360aa',
        'InstanceType': 'm1.small',
    })
)



# Copyright (c) 2013 MetaMetrics, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

"""Python wrappers for CloudFormation intrinsic functions

These are all available without preamble in a pyplate's global namespace.

These help make the pyplate look a little more like python than JSON, and can
be ignored if you want to write the raw JSON directly. (But you don't want
that, right? After all, that's why you're using pyplates.)

Notes:

* All "Return" values are a little midleading, in that what really gets
  returned is the JSON-ified CFN intrinsic function. When using these in
  pyplates, though, it's probably easiest to forget that and only be
  concerned with how CFN will handle this, hence the Return values.
* To avoid the issue of using the wrong types with functions that take
  sequences as input (join, select), argument unpacking is used. Therefore,
  pass the sequence elements one at a time, rather than the sequence itself,
  after passing the separator (for join) or index (for select).
* Using CloudFormation's Join function versus a pythonic 'string'.join
  allows you to use CFN's intrinsic functions inside a join statement. The
  pythonic string join method may literally interpret a call to an intrinsic
  function, causing the resulting JSON to be interpreted as a string and
  ignored by the CloudFormation template parser
* Functions related to conditions tend to overlap with python keywords, so they
  are prefixed with ``c_`` to differentiate them (so, Fn::And become ``c_and``)

.. note:
    Documentation for the functions is verbatim from the AWS Docs,
    except where identifiers are changed to fit with normal python style.

    Links:
    * `Intrinsic Functions <cfn-functions_>`_
    * `Intrinsic Condition Functions <cfn-functions-conditions_>`_


"""

# Where needed, exception error messages are stored on the intrinsic function
# wrappers to make testing the function failure cases very easy


from exceptions import IntrinsicFuncInputError

__all__ = [
    'base64',
    'find_in_map',
    'get_att',
    'get_azs',
    'join',
    'select',
    'ref',
    'c_ref',
    'c_and',
    'c_or',
    'c_not',
    'c_if',
    'c_equals',
]


def base64(value):
    """The intrinsic function Fn::Base64 returns the Base64 representation of \
    the input string.

    This function is typically used to pass encoded data to
    Amazon EC2 instances by way of the UserData property.

    Args:
        value: The string value you want to convert to Base64

    Returns: The original string, in Base64 representation
    """
    return {'Fn::Base64': value}


def find_in_map(map_name, key, value):
    """The intrinsic function Fn::FindInMap returns the value of a key from a \
    mapping declared in the Mappings section.

    Args:
        map_name: The logical name of the mapping declared in the Mappings
            section that contains the key-value pair.
        key: The name of the mapping key whose value you want.
        value: The value for the named mapping key.

    Returns: The map value.

    .. note:
        find_in_map is represented here "just in case".
        Most likely, using the python equivalent in a pyplate will be
        easier to both look at and maintain.

    """
    return {'Fn::FindInMap': [map_name, key, value]}


def get_att(logical_name, attribute):
    """The intrinsic function Fn:GetAtt returns the value of an attribute from \
    a resource in the template.

    Args:
        logical_name: The logical name of the resource that
            contains the attribute you want.
        attribute: The name of the resource-specific attribute
            whose value you want. See the resource's reference




 page
            [#cfn-resources] for details about the attributes available
            for that resource type.

    Returns: The attribute value.

    """
    return {'Fn::GetAtt': [logical_name, attribute]}


def get_azs(region=''):
    """The intrinsic function Fn::GetAZs returns an array that lists all \
    Availability Zones for the specified region.

    Because customers have access to different Availability Zones, the
    intrinsic function Fn::GetAZs enables template authors to write
    templates that adapt to the calling user's access. This frees you from
    having to hard-code a full list of Availability Zones for a specified
    region.

    Args:
        region: The name of the region for which you want to get the
            Availability Zones.
            You can use the AWS::Region pseudo parameter to specify the region
            in which the stack is created. Specifying an empty string is
            equivalent to specifying AWS::Region.
    Returns: The list of Availability Zones for the region.

    """
    return {'Fn::GetAZs': region}


def join(sep, *args):
    """The intrinsic function Fn::Join appends a set of values into a single \
    value, separated by the specified delimiter.

    If a delimiter is the empty string, the set of values are
    concatenated with no delimiter.

    Args:
        delimiter: The value you want to occur between fragments. The delimiter
            will occur between fragments only.
            It will not terminate the final value.

        *args: Any number of values you want combined, passed as
            positional arguments

    Returns: The combined string.

    """
    if len(args) < 2:
        raise IntrinsicFuncInputError(join._errmsg_needinput)

    return {'Fn::Join': [sep, list(args)]}

join._errmsg_needinput = 'Unable to join on one or less things!'


def select(index, *args):
    """The intrinsic function Fn::Select returns a single object from a list of objects by index.

    .. note:
        select is represented here "just in case".
        Most likely, using the python equivalent in a pyplate will be
        easier to both look at and maintain, but in the event that selects
        need to take place after CFN has interpolated all the intrinsic
        functions, it may still be useful.

    .. warning:: Important
        Fn::Select does not check for null values or if the index is out of
        bounds of the array. Both conditions will result in a stack error,
        so you should be certain that the index you choose is valid, and that
        the list contains non-null values.

    Args:
        index: The index of the object to retrieve. This must be a value
            from zero to N-1, where N represents the number of
            elements in the array.
        *args: Any number of objects to select from, passed as
            positional arguments. None of the arguments can be ``None``

    Returns: The selected object.

    """
    try:
        index = int(index)
    except ValueError:
        raise IntrinsicFuncInputError(select._errmsg_int)
    if not args:
        raise IntrinsicFuncInputError(select._errmsg_empty)
    if filter(lambda x: x is None, args):
        raise IntrinsicFuncInputError(select._errmsg_null)
    try:
        args[index]
    except IndexError:
        raise IntrinsicFuncInputError(select._errmsg_index)

    return {'Fn::Select': [index, list(args)]}

select._errmsg_int = 'Index must be a number!'
select._errmsg_empty = 'Unable to select from an empty list!'
select._errmsg_null = 'List of selections include null values!'
select._errmsg_index = 'Provided index is invalid!'


def ref(logical_name):
    """The intrinsic function Ref returns the value of the specified parameter or resource.

    When you are declaring a resource in a template and you need to
    specify another template resource by name, you can use the Ref to refer
    to that other resource. In general, Ref returns the name of the
    resource. For example, a reference to an
    AWS::AutoScaling::AutoScalingGroup returns the name of that Auto
    Scaling group resource.

    For some resources, an identifier is returned that has another
    significant meaning in the context of the resource. An AWS::EC2::EIP
    resource, for instance, returns the IP address, and an
    AWS::EC2::Instance returns the instance ID.

    Args:
        logical_name: The logical name of the resource or parameter you want
            to dereference.

    Returns:
        * When you specify a parameter's logical name, it returns the value
            of the parameter.
        * When you specify a resource's logical name, it returns a value
            that you can typically use to refer to that resource.

    .. note:: You can also use Ref to add values to Output messages.

    """
    return {'Ref': logical_name}


def c_ref(condition_name):
    """The intrinsic function Condition used to reference a named condition

    When you refer to a condition in another condition or associate the
    condition with a resource, you use the Condition: key. For the Fn::If
    function, you only need to specify the condition name.

    Args:
        condition_name: The name of the condition you want to reference.

    Returns:
        * A reference to the named condition

    """
    return {'Condition': condition_name}


def _validate_logical_condition_counts(fn, conditions):
    # c_and / c_or have these limits applied, this keeps it DRY
    if len(conditions) < 2:
        raise IntrinsicFuncInputError(fn._errmsg_min)
    elif len(conditions) > 10:
        raise IntrinsicFuncInputError(fn._errmsg_max)


def c_and(*conditions):
    """The intrinsic conditional function Fn::And

    Returns true (for the pruposes of the cfn template) if all the specified conditions
    evaluate to true, or returns false if any one of the conditions evaluates to false.

    Fn::And acts as an AND operator.

    The minimum number of conditions that you can include is 2, and the maximum is 10.

    Args:
        *conditions: The conditions to evaluate

    """
    _validate_logical_condition_counts(c_and, conditions)
    return {'Fn::And': list(conditions)}
c_and._errmsg_min = "Minimum umber of conditions for 'c_and' condition is 2"
c_and._errmsg_max = "Maximum umber of conditions for 'c_and' condition is 10"


def c_or(*conditions):
    """The intrinsic conditional function Fn::Or

    Returns true (for the pruposes of the cfn template) if any one of the specified conditions
    evaluate to true, or returns false if all of the conditions evaluates to false.

    Fn::Or acts as an OR operator.

    The minimum number of conditions that you can include is 2, and the maximum is 10.

    Args:
        *conditions: The conditions to evaluate

    """
    _validate_logical_condition_counts(c_or, conditions)
    return {'Fn::Or': list(conditions)}
c_or._errmsg_min = "Minimum umber of conditions for 'c_or' condition is 2"
c_or._errmsg_max = "Maximum umber of conditions for 'c_or' condition is 10"


def c_not(condition):
    """The intrinsic conditional function Fn::Not

    Returns true for a condition that evaluates to false or
    returns false for a condition that evaluates to true.

    Fn::Not acts as a NOT operator.

    """
    return {'Fn::Not': [condition]}


def c_equals(value_1, value_2):
    """The intrinsic conditional function Fn::Equals

    Compares if two values are equal.

    Returns true if the two values are equal or false if they aren't.

    """
    return {'Fn::Equals': [value_1, value_2]}


def c_if(condition_name, value_if_true, value_if_false):
    """The intrinsic conditional function Fn::If representsa a ternary conditional operator

    Returns one value if the specified condition evaluates to true and another value
    if the specified condition evaluates to false.

    Currently, AWS CloudFormation supports the Fn::If intrinsic function in the
    metadata attribute, update policy attribute, and property values in the Resources
    section and Outputs sections of a template.

    You can use the AWS::NoValue pseudo parameter as a return value
    to remove the corresponding property.

    """
    return {'Fn::If': [condition_name, value_if_true, value_if_false]}



# Copyright (c) 2013 MetaMetrics, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.


class Error(Exception):
    """Base exception class for cfn_pyplates

    A namespaced Exception subclass with explicit 'message' support.
    Will be handled at template generation, with the message being delivered
    to the user.

    Args:
        message: An optional message to package with the Error
        args: Any number of optional arguments, to be used as subclasses
            see fit.

    """

    message = 'An unknown error has occurred.'

    def __init__(self, message=None, *args):
        if not message:
            # This is a subclass, message is a static attr
            message = self.message
        else:
            # Error is being directly instantiated, set the message
            self.message = message
        self.args = (message,) + args


class AddRemoveError(Error):
    """Raised when attempting to attach weird things to a JSONableDict

    Weird things, in this case, mean anything that isn't a JSONableDict

    Args:
        message: An optional message to package with the Error

    """

    message = 'Only subclasses of JSONableDict can be added or removed'


class IntrinsicFuncInputError(Error):
    """Raised when passing bad input values to an intrinsic function

    Args:
        message: An optional message to package with the Error

    """

    message = 'Invalid arguments passed to intrinsic function'



# Copyright (c) 2013 MetaMetrics, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

import unittest

from cfn_pyplates import exceptions, functions


class IntrinsicFuncsTestCase(unittest.TestCase):
    def test_base64(self):
        ret = functions.base64('test')
        self.assertEqual(ret['Fn::Base64'], 'test')

    def test_find_in_map(self):
        ret = functions.find_in_map('MapName', 'MapKey', 'MapValue')
        self.assertEqual(ret['Fn::FindInMap'], ['MapName', 'MapKey', 'MapValue'])

    def test_get_att(self):
        ret = functions.get_att('ThingName', 'AttrName')
        self.assertEqual(ret['Fn::GetAtt'], ['ThingName', 'AttrName'])

    def test_get_azs(self):
        ret = functions.get_azs('region')
        self.assertEqual(ret['Fn::GetAZs'], 'region')

    def test_join(self):
        ret = functions.join('.', 'x', 'y', 'z')
        self.assertEqual(ret['Fn::Join'], ['.', ['x', 'y', 'z']])

    def test_select(self):
        ret = functions.select(3, 1, 2, 3, 4, 5)
        self.assertEqual(ret['Fn::Select'], [3, [1, 2, 3, 4, 5]])

    def test_ref(self):
        ret = functions.ref('ThingName')
        self.assertEqual(ret['Ref'], 'ThingName')

    def test_and(self):
        ret = functions.c_and('Thing 1', 'Thing 2')
        self.assertEqual(ret['Fn::And'], ['Thing 1', 'Thing 2'])

    def test_or(self):
        ret = functions.c_or('Thing 1', 'Thing 2')
        self.assertEqual(ret['Fn::Or'], ['Thing 1', 'Thing 2'])

    def test_not(self):
        ret = functions.c_not('Thing')
        self.assertEqual(ret['Fn::Not'], ['Thing'])

    def test_if(self):
        ret = functions.c_if('If', "Condition 1", "Condition 2")
        self.assertEqual(ret['Fn::If'], ['If', 'Condition 1', 'Condition 2'])

    def test_equals(self):
        ret = functions.c_equals('Thing 1', 'Thing 2')
        self.assertEqual(ret['Fn::Equals'], ['Thing 1', 'Thing 2'])


class IntrinsicFuncsFailureCase(unittest.TestCase):
    def test_join_unjoinable(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.join('.')
        self.assertEqual(ctx.exception.message,
            functions.join._errmsg_needinput)

        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.join('.', 'x')
        self.assertEqual(ctx.exception.message,
            functions.join._errmsg_needinput)

    def test_select_int(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.select('not an int')
        self.assertEqual(ctx.exception.message,
            functions.select._errmsg_int)

    def test_select_empty(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.select(0)
        self.assertEqual(ctx.exception.message,
            functions.select._errmsg_empty)

    def test_select_null(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.select(0, '', None)
        self.assertEqual(ctx.exception.message,
            functions.select._errmsg_null)

    def test_select_index(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.select(2, '', '')
        self.assertEqual(ctx.exception.message,
            functions.select._errmsg_index)

    def test_and_min(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.c_and('1 Condition')
        self.assertEqual(ctx.exception.message,
            functions.c_and._errmsg_min)

    def test_and_max(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.c_and(*['%d Condition' % i for i in range(20)])
        self.assertEqual(ctx.exception.message,
            functions.c_and._errmsg_max)

    def test_or_min(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.c_or('1 Condition')
        self.assertEqual(ctx.exception.message,
            functions.c_or._errmsg_min)

    def test_or_max(self):
        with self.assertRaises(exceptions.IntrinsicFuncInputError) as ctx:
            functions.c_or(*['%d Condition' % i for i in range(20)])
        self.assertEqual(ctx.exception.message,
            functions.c_or._errmsg_max)



cft = CloudFormationTemplate(description="A slightly more useful template.")

cft.resources.add(
    Resource('MyS3Bucket', 'AWS::S3::Bucket', None, Metadata(
        {"Object1": "Location1", "Object2": "Location2"}
    ))
)



# Copyright (c) 2013 MetaMetrics, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

'''pyplates: CloudFormation templates, generated with python

See also:

- https://aws.amazon.com/cloudformation/
- https://cfn-pyplates.readthedocs.org/ (you might already be here)
- https://github.com/seandst/cfn-pyplates/
'''
# inject modules into the cfn_pyplates namespace
from cfn_pyplates import cli, core, exceptions, functions, options  # NOQA



import sys

# If our base template isn't on the PYTHONPATH already, we need to do this:
sys.path.append('../path/to/base/templates')

import basetemplate

class InheritingTemplate(basetemplate.BaseTemplate):
    def add_resources(self):
        self.add_server()
        self.add_bucket()

cft = InheritingTemplate("Our usual project", options)
cft.add_resources()




cft = CloudFormationTemplate(description="A self-referential template.")

cft.parameters.add(Parameter('ImageId', 'String',
    {
        'Default': 'ami-c30360aa',
        'Description': 'Amazon Machine Image ID to use for the created instance',
        'AllowedPattern': 'ami-[a-f0-9]+',
        'ConstraintDescription': 'must start with "ami-" followed by lowercase hexidecimal characters',
    })
)

cft.parameters.add(Parameter('InstanceName', 'String',
    {
        'Description': 'A name for the instance to be created',
    })
)

cft.parameters.add(Parameter('InstanceType', 'String',
    {
        'Default': 'm1.small',
    })
)

# Now the Resource definition is all refs, totally customizeable at
# stack creation
cft.resources.add(Resource('AnInstance', 'AWS::EC2::Instance',
    {
        'ImageId': ref('ImageId'),
        'InstanceType': ref('InstanceType'),
        'Tags': ec2_tags({
            'Name': ref('InstanceName'),
        })
    })
)

cft.outputs.add(Output('DnsName',
    get_att('AnInstance', 'PublicDnsName'),
    'The public DNS Name for AnInstance')
)



class ProjectTemplate(CloudFormationTemplate):

    def add_resources(self):
        self.add_server()
        self.add_bucket()

    def add_server(self):
        self.parameters.add(
            Parameter('EC2InstanceType', 'String',
                {
                    'Default': 'm1.small',
                    'Description': 'Instance type to use for created Server EC2 instance',
                    'AllowedPattern': 'm3.[a-z]+',
                    'ConstraintDescription': 'Must use one of the m3 instance types.',
                }
            )
        )
        self.resources.add(
            Resource('Server', 'AWS::EC2::Instance',
                {
                    'ImageId': 'ami-c30360aa',
                    'InstanceType': ref('EC2InstanceType')
                }
            )
        )

    def add_bucket(self):
        self.resource.add(
            Resource('StaticFiles', 'AWS::S3::Bucket', {'AccessControl: PublicRead'}, Metadata(
                {'Object1': 'Location1', 'Object2': 'Location2'}
            ))
        )

cft = ProjectTemplate(description='My project template.')
cft.add_resources()



# Copyright (c) 2013 MetaMetrics, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

"""CLI Entry points for handy bins

Documentation for CLI methods defined in this file will be that method's
usage information as seen on the command-line.

"""
import sys

import yaml
from docopt import docopt
from schema import Schema, Use, Or

from cfn_pyplates import core
from cfn_pyplates.options import OptionsMapping


def _open_outfile(outfile_name):
    'Helper function for Schema to open the outfile'
    if outfile_name == '-':
        return sys.stdout
    else:
        return open(outfile_name, 'w')


def _open_optionfile(optionfile_name):
    'Helper function for Schema to open the option file'
    if optionfile_name == '-':
        return sys.stdin
    else:
        return open(optionfile_name)


def generate():
    """Generate CloudFormation JSON Template based on a Pyplate

Usage:
  cfn_py_generate <pyplate> [<outfile>] [-o/--options=<options_mapping>]
  cfn_py_generate (-h|--help)
  cfn_py_generate --version

Arguments:
  pyplate
    Input pyplate file name

  outfile
    File in which to place the compiled JSON template
    (if omitted or '-', outputs to stdout)

Options:
  -o --options=<options_mapping>
    Input JSON or YAML file for options mapping
    exposed in the pyplate as "options_mapping"
    (if '-', accepts input from stdin)

  -h --help
    This usage information

WARNING!

  Do not use pyplates that you haven't personally examined!

  A pyplate is a crazy hybrid of JSON-looking python.
  exec is used to read the pyplate, so any code in there is going to
  run, even potentailly harmful things.

  Be careful.

    """
    from pkg_resources import require
    version = require("cfn-pyplates")[0].version
    args = docopt(generate.__doc__, version=version)
    scheme = Schema({
        '<pyplate>': Use(open),
        '<outfile>': Or(None, Use(_open_outfile)),
        '--options': Or(None, Use(_open_optionfile)),
        '--help': Or(True, False),
        '--version': Or(True, False),
    })
    args = scheme.validate(args)

    options_file = args['--options']
    if options_file:
        options = yaml.load(options_file)
    else:
        options = {}

    output = core.generate_pyplate(args['<pyplate>'], OptionsMapping(options))

    if not output:
        return 1

    if not args['<outfile>'] or args['<outfile>'] == '-':
        print output
    else:
        args['<outfile>'].write(output)

    # Explicitly return a posixy "EVERYTHING IS OKAY" 0
    return 0



# Copyright (c) 2013 MetaMetrics, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
import json
import unittest
from textwrap import dedent
from tempfile import NamedTemporaryFile

from cfn_pyplates import core


class TestResource(core.JSONableDict):
    pass


class JSONableDictTestCase(unittest.TestCase):
    def test_crazy_addchild_ordering(self):
        # We should be able to add children to any element at any time,
        # before or after that element has been added to a parent.
        bm = core.JSONableDict()
        tr1 = TestResource({'Id': 1})
        tr2 = TestResource({'Id': 2})
        # Tack the TestResource onto the template
        bm.add(tr1)
        # .add should return the added resource
        added_resource = bm.add(tr1)
        self.assertIs(added_resource, tr1)
        # Add a child to tr1 *after* adding it to bm
        tr1.add(tr2)
        # Then make sure tr2 is visible from the template
        self.assertIn('TestResource', bm['TestResource'])
        # Double check Ids...
        self.assertEqual(bm['TestResource']['Id'], 1)
        self.assertEqual(bm['TestResource']['TestResource']['Id'], 2)

        # Now mess with the nested child, and make sure those changes
        # are still seen in the template
        tr2.update({'This Is': 'A Test'})
        self.assertEqual(bm['TestResource']['TestResource']['This Is'], 'A Test')

    def test_contructor_dict_arg(self):
        # Making sure that a resource instantiated with a params dict is updated correctly
        update_dict = {'Test': 'A custom custructydict'}
        bm = core.JSONableDict()
        bm.add(TestResource(update_dict))
        self.assertEqual(bm['TestResource'], update_dict)

    def test_naming(self):
        obj = TestResource(name='NormalName')
        # Getter method works?
        self.assertEqual(obj.name, 'NormalName')
        self.assertEqual(obj._name, 'NormalName')

        # Setter method works?
        obj.name = 'DifferentName'
        self.assertEqual(obj.name, 'DifferentName')
        self.assertEqual(obj._name, 'DifferentName')

        # Del method works?
        del(obj.name)
        self.assertEqual(obj.name, 'TestResource')
        self.assertIsNone(obj._name)

    def test_getsetattr(self):
        bm = core.JSONableDict()

        self.assertNotIn('TestResource', bm)
        bm.test_resource = TestResource()
        self.assertIn('TestResource', bm)
        del(bm.test_resource)
        self.assertNotIn('TestResource', bm)


class CloudFormationTemplateTestCase(unittest.TestCase):
    def test_has_template_attrs(self):
        description = 'Test Description!'
        cft = core.CloudFormationTemplate(description)
        self.assertEqual(cft['Description'], description)

        self.assertIn('Parameters', cft)
        self.assertIsInstance(cft.parameters, core.Parameters)

        self.assertIn('Mappings', cft)
        self.assertIsInstance(cft.mappings, core.Mappings)

        self.assertIn('Resources', cft)
        self.assertIsInstance(cft.resources, core.Resources)

        self.assertIn('Outputs', cft)
        self.assertIsInstance(cft.outputs, core.Outputs)

    def test_no_description(self):
        cft = core.CloudFormationTemplate()
        self.assertNotIn('Description', cft)

    def test_jsonification(self):
        cft = core.CloudFormationTemplate('This is a test')
        cft.parameters.update({'These': 'are awesome!'})

        # We expect to see the AWSTemplateFormatVersion, the provided
        # description, and the provided parameters. We do not expect to
        # see empty Mappings, Resources, or Outputs.
        expected_out = dedent(u'''\
        {
          "AWSTemplateFormatVersion": "2010-09-09",
          "Description": "This is a test",
          "Parameters": {
            "These": "are awesome!"
          }
        }''')
        self.assertEqual(unicode(cft), expected_out)


class ResourcesTestCase(unittest.TestCase):
    def test_resource(self):
        # No properties for this one, just make sure the resource comes
        # out right
        cft = core.CloudFormationTemplate()
        cft.resources.test = core.Resource('TestResource', 'AWS::Resource::Test')

        # Should have a new 'TestResource' key in our template resources
        self.assertIn('TestResource', cft.resources)

        # And it should look like this...
        expected_out = dedent(u'''\
        {
          "Type": "AWS::Resource::Test"
        }''')
        self.assertEqual(unicode(cft.resources.test), expected_out)

    def test_resource_with_properties(self):
        properties_dict = {'Key1': 'Value1', 'Key2': 'Value2'}
        properties = core.Properties(properties_dict)
        res = core.Resource('TestResource', 'AWS::Resource::Test', properties)
        cft = core.CloudFormationTemplate()
        cft.resources.test = res

        # The output should have the properties attached
        expected_out = dedent(u'''\
        {
          "Type": "AWS::Resource::Test",
          "Properties": {
            "Key2": "Value2",
            "Key1": "Value1"
          }
        }''')
        self.assertEqual(unicode(cft.resources.test), expected_out)

    def test_resource_with_metadata(self):
        metadata = core.Metadata({"Object1": "Location1", "Object2": "Location2"})
        res = core.Resource('TestResource', 'AWS::Resource::Test', None, metadata)
        cft = core.CloudFormationTemplate()
        cft.resources.test = res

        # The output should have the metadata attached
        expected_out = dedent(u'''\
        {
          "Type": "AWS::Resource::Test",
          "Metadata": {
            "Object1": "Location1",
            "Object2": "Location2"
          }
        }''')
        self.assertEqual(unicode(cft.resources.test), expected_out)

    def test_resource_with_deletion_policy(self):
        deletion_policy = core.DeletionPolicy("Retain")
        res = core.Resource('TestResource', 'AWS::Resource::Test', None, deletion_policy)
        cft = core.CloudFormationTemplate()
        cft.resources.test = res

        # The output should have the metadata attached
        expected_out = dedent(u'''\
        {
          "Type": "AWS::Resource::Test",
          "DeletionPolicy": "Retain"
        }''')
        self.assertEqual(unicode(cft.resources.test), expected_out)

    def test_resource_with_depends_on(self):
        depends_on = core.DependsOn("Location2")
        res = core.Resource('TestResource', 'AWS::Resource::Test', None, depends_on)
        cft = core.CloudFormationTemplate()
        cft.resources.test = res

        # The output should have the metadata attached
        expected_out = dedent(u'''\
        {
          "Type": "AWS::Resource::Test",
          "DependsOn": "Location2"
        }''')
        self.assertEqual(unicode(cft.resources.test), expected_out)

    def test_resource_with_update_policy(self):
        update_policy = core.UpdatePolicy(
            {"MaxBatchSize": "Location1", "MinInstancesInService": "Location2", "PauseTime": "30"})
        res = core.Resource('TestResource', 'AWS::Resource::Test', None, update_policy)
        cft = core.CloudFormationTemplate()
        cft.resources.test = res

        # The output should have the metadata attached
        expected_out = dedent(u'''\
        {
          "Type": "AWS::Resource::Test",
          "UpdatePolicy": {
            "PauseTime": "30",
            "MaxBatchSize": "Location1",
            "MinInstancesInService": "Location2"
          }
        }''')
        self.assertEqual(unicode(cft.resources.test), expected_out)

    def test_resource_with_extended_attributes(self):
        update_policy = core.UpdatePolicy({"Object1": "Location1", "Object2": "Location2"})
        metadata = core.Metadata({"Object1": "Location1", "Object2": "Location2"})
        deletion_policy = core.DeletionPolicy("Retain")
        depends_on = core.DependsOn("Location2")
        res = core.Resource('TestResource', 'AWS::Resource::Test', None,
                            [metadata, update_policy, deletion_policy, depends_on])
        cft = core.CloudFormationTemplate()
        cft.resources.test = res

        # The output should have the metadata attached
        expected_out = dedent(u'''\
        {
          "Type": "AWS::Resource::Test",
          "Metadata": {
            "Object1": "Location1",
            "Object2": "Location2"
          },
          "UpdatePolicy": {
            "Object1": "Location1",
            "Object2": "Location2"
          },
          "DeletionPolicy": "Retain",
          "DependsOn": "Location2"
        }''')
        self.assertEqual(unicode(cft.resources.test), expected_out)

    def test_resource_with_condition(self):
        condition = core.Condition('TestCondition', {'Fn::Fake': 'ConditionValue'})
        res = core.Resource('TestResource', 'AWS::Resource::Test', None, condition)
        cft = core.CloudFormationTemplate()
        cft.resources.test = res

        # The output should have the metadata attached
        expected_out = dedent(u'''\
        {
          "Type": "AWS::Resource::Test",
          "Condition": "TestCondition"
        }''')
        self.assertEqual(unicode(cft.resources.test), expected_out)


class ConditionsTestCase(unittest.TestCase):
    def test_condition(self):
        cft = core.CloudFormationTemplate()
        cft.conditions.test = core.Condition('TestCondition', {'Fn::Fake': 'ConditionValue'})

        # Should have a new 'TestCondition' key in our template resources
        self.assertIn('TestCondition', cft.conditions)

        # And it should look like this...
        expected_out = dedent(u'''\
        {
          "Fn::Fake": "ConditionValue"
        }''')
        self.assertEqual(unicode(cft.conditions.test), expected_out)

    def test_condition_ref(self):
        cft = core.CloudFormationTemplate()
        cft.conditions.test = core.Condition('TestCondition', {'Ref': 'ReferencedThing'})

        # And it should look like this...
        expected_out = dedent(u'''\
        {
          "Ref": "ReferencedThing"
        }''')
        self.assertEqual(unicode(cft.conditions.test), expected_out)


class MiscElementsTestCase(unittest.TestCase):
    def test_mapping_element(self):
        mappings = core.Mappings()

        mapping_letters = core.Mapping('MappingKeyLetters',
            {
                'keya': 'valuea',
                'keyb': 'valueb'
            }
        )
        mapping_numbers = core.Mapping('MappingKeyNumbers',
            {
                'key1': 'value1',
                'key2': 'value2'
            }
        )

        mappings.add(mapping_letters)
        mappings.add(mapping_numbers)

        self.assertEqual(mappings['MappingKeyLetters']['keya'], 'valuea')
        self.assertEqual(mappings['MappingKeyLetters']['keyb'], 'valueb')
        self.assertEqual(mappings['MappingKeyNumbers']['key1'], 'value1')
        self.assertEqual(mappings['MappingKeyNumbers']['key2'], 'value2')

    def test_resource_element(self):
        resource = core.Resource('ResourceName',
            'AWS::Testing::ResourceType',
            {
                'Key1': 'Value1',
                'Key2': 'Value2',
            }
        )
        self.assertEqual(resource.name, 'ResourceName')
        self.assertIn('Properties', resource)
        properties = resource['Properties']
        self.assertEqual(properties['Key1'], 'Value1')
        self.assertEqual(properties['Key2'], 'Value2')

    def test_properties(self):
        properties = core.Properties({
            'Key1': 'Value1',
            'Key2': 'Value2',
        })
        # Create a resource, and test attaching properties to it
        resource = core.Resource('ResourceName',
            'AWS::Testing::ResourceType')
        resource.add(properties)

        self.assertEqual(resource.name, 'ResourceName')
        self.assertIn('Properties', resource)
        properties = resource['Properties']
        self.assertEqual(resource['Properties']['Key1'], 'Value1')
        self.assertEqual(resource['Properties']['Key2'], 'Value2')

    def test_parameter_element(self):
        parameter = core.Parameter('ParameterName',
            'String',
            {'Key': 'Value'})

        self.assertEqual(parameter.name, 'ParameterName')
        self.assertEqual(parameter['Type'], 'String')
        self.assertEqual(parameter['Key'], 'Value')

    def test_output_element(self):
        output = core.Output('OutputName',
            'Output Value',
            'Output Description')
        self.assertEqual(output.name, 'OutputName')
        self.assertEqual(output['Value'], 'Output Value')
        self.assertEqual(output['Description'], 'Output Description')

    def test_ec2_tags_element(self):
        tags = core.ec2_tags({
            'TestKey1': 'TestValue1',
            'TestKey2': 'TestValue2'
        })
        for entry in tags:
            self.assertIn('Key', entry)
            self.assertIn('Value', entry)
            if entry['Key'] == 'TestKey1':
                self.assertEqual(entry['Value'], 'TestValue1')
            if entry['Key'] == 'TestKey2':
                self.assertEqual(entry['Value'], 'TestValue2')


class GenerateTestCase(unittest.TestCase):
    def test_callable_generate(self):
        # Make a pyplate that uses the options mapping
        pyplate_contents = dedent(u'''\
        cft = CloudFormationTemplate('This is a test')
        cft.parameters.update({
            'Exists': options['ThisKeyExists']
        })''')
        pyplate = NamedTemporaryFile()
        pyplate.write(pyplate_contents)
        pyplate.flush()

        # call generate directly with an options mapping, no CLI
        output = json.loads(core.generate_pyplate(pyplate.name, {'ThisKeyExists': True}))
        self.assertTrue(output['Parameters']['Exists'])



from pyplates import core, functions

class BaseTemplate(core.CloudFormationTemplate):

    def add_server(self):
        self.parameters.add(
            core.Parameter('EC2InstanceType', 'String',
                {
                    'Default': 'm1.small',
                    'Description': 'Instance type to use for created Server EC2 instance',
                    'AllowedPattern': 'm3.[a-z]+',
                    'ConstraintDescription': 'Must use one of the m3 instance types.',
                }
            )
        )
        self.resources.add(
            core.Resource('Server', 'AWS::EC2::Instance',
                {
                    'ImageId': self.options['AmiId'],
                    'InstanceType': functions.ref('EC2InstanceType')
                }
            )
        )

    def add_bucket(self):
        self.resource.add(
            core.Resource(
                'StaticFiles',
                'AWS::S3::Bucket',
                {'AccessControl: PublicRead'},
                Metadata({'Object1': 'Location1', 'Object2': 'Location2'})
            )
        )



# Copyright (c) 2013 MetaMetrics, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

from cStringIO import StringIO
from textwrap import dedent
from tempfile import NamedTemporaryFile
import json
import sys
import unittest

from cfn_pyplates import cli

try:
    from mock import patch
    mock_error = None
except ImportError:
    patch = None
    mock_error = 'skipped -- install mock to run this test'


@unittest.skipIf(patch is None, mock_error)
class CLITestCase(unittest.TestCase):
    def setUp(self):  # NOQA
        # Patch out argv, stdin, and stdout so that we can do some
        # useful things, like:
        # - Fake arguments to be used by a CLI function
        # - Write to stding, simulating user input
        # - Suppress stdout to hide prompts during the test run
        argv_patcher = patch('sys.argv')
        argv_patcher.start()
        self.addCleanup(argv_patcher.stop)
        stdin_patcher = patch('sys.stdin', new=StringIO())
        stdin_patcher.start()
        self.addCleanup(stdin_patcher.stop)
        stdout_patcher = patch('sys.stdout', new=StringIO())
        stdout_patcher.start()
        self.addCleanup(stdout_patcher.stop)

    def _make_pyplate(self, contents):
        contents = dedent(contents)
        pyplate = NamedTemporaryFile()
        pyplate.write(contents)
        pyplate.flush()
        return pyplate

    def _generate(self, fail_on_error=True):
        return_code = cli.generate()
        sys.stdout.seek(0)
        out = '\n'.join([line for line in sys.stdout.readlines()])
        if return_code != 0:
            if fail_on_error:
                self.fail('generate failed, stdout dump follows:\n{0}'.format(
                    out)
                )
        return out

    def test_generate(self):
        # Make a pyplate that uses the options mapping
        pyplate = self._make_pyplate(u'''\
        cft = CloudFormationTemplate('This is a test')
        cft.parameters.update({
            'Exists': options['ThisKeyExists'],
            'DoesNotExist': options['ThisKeyDoesNotExist']
        })''')

        # Now make an options mapping with only one of those keys in it
        # to simultaneously test options interpolation and
        # user-prompted input
        options_mapping_contents = dedent(u'''\
        {
            'ThisKeyExists': true
        }
        ''')
        options_mapping = NamedTemporaryFile()
        options_mapping.write(options_mapping_contents)
        options_mapping.flush()

        # The outfile which will receive the rendered json
        outfile = NamedTemporaryFile()

        # Populate sys.argv with something reasonable based on all the
        # tempfiles. On the command line this would look like
        # "cfn_py_generate pyplate outfile -o options_mapping"
        sys.argv = ['cfn_py_generate', pyplate.name, outfile.name,
            '-o', options_mapping.name]
        # Prime stdin with the answer to our interactive question
        input = 'Test'
        sys.stdin.write('{0}\n'.format(input))
        sys.stdin.seek(0)

        # Run the command, catch it if it tries to exit the interpreter
        self._generate()

        # Load the template back into python for assertions
        template = json.load(outfile)

        # The Exists parameter should evaluate to bool True...
        # If so, then options_mapping interpolation works
        self.assertTrue(template['Parameters']['Exists'])

        # The DoesNotExist parameter should be what was injected to stdin
        # If so, then prompts to populate missing options_mapping entries work
        self.assertEqual(template['Parameters']['DoesNotExist'], input)

    def test_generate_no_options_no_outfile(self):
        # generate with no options mapping to stdout
        description = 'This is a test.'
        pyplate = self._make_pyplate("cft = CloudFormationTemplate('{}')".format(description))

        # Populate sys.argv with something reasonable based on all the
        # tempfiles. On the command line this would look like
        # "cfn_py_generate pyplate outfile -o options_mapping"
        sys.argv = ['cfn_py_generate', pyplate.name, '-']

        template = json.loads(self._generate())

        # The description in the template should match the one in the pyplate
        self.assertEqual(template['Description'], description)

    def test_generate_stdin_options(self):
        # Make a pyplate that uses the options mapping
        pyplate = self._make_pyplate(u'''\
        cft = CloudFormationTemplate('This is a test')
        cft.parameters.update({
            'Exists': options['ThisKeyExists'],
        })''')

        # Prime stdin with the contents of the options file
        sys.stdin.write('{0}\n'.format(dedent(u'''\
        {
            'ThisKeyExists': true
        }
        ''')))
        sys.stdin.seek(0)

        # Populate sys.argv with something reasonable based on all the
        # tempfiles. On the command line this would look like
        # "cfn_py_generate pyplate outfile -o options_mapping"
        sys.argv = ['cfn_py_generate', pyplate.name, '-o', '-']

        template = json.loads(self._generate())

        # The Exists parameter should evaluate to bool True...
        # If so, then options_mapping interpolation works
        self.assertTrue(template['Parameters']['Exists'])

    def test_broken_pyplate(self):
        pyplate = self._make_pyplate(u'''\
        I am a broken pyplate.
        })''')
        sys.argv = ['cfn_py_generate', pyplate.name]
        out = self._generate(fail_on_error=False)
        # This pyplate should fail because its python syntax isn't valid
        self.assertIn('SyntaxError', out)



description = 'My static webapp {0} stack'.format(options['StackRole'])
cft = CloudFormationTemplate(description)

# Make a load balancer
cft.resources.add(Resource('LoadBalancer',
    'AWS::ElasticLoadBalancing::LoadBalancer',
    {
        'AvailabilityZones': options['AppServerAvailabilityZones'],
        'HealthCheck': {
            'HealthyThreshold': '2',
            'Interval': '30',
            'Target': 'HTTP:80/',
            'Timeout': '5',
            'UnhealthyThreshold': '2'
        },
        'Listeners': [
            {
                'LoadBalancerPort': '80',
                'Protocol': 'HTTP',
                'InstanceProtocol': 'HTTP',
                'InstancePort': '80',
                'PolicyNames': []
            },
        ],
    })
)

# Make a security group for the load balancer
cft.resources.add(Resource('ELBSecurityGroup',
    'AWS::EC2::SecurityGroup',
    {
        'GroupDescription': 'allow traffic from our app servers to the load balancer'
    })
)

# Put an ingress policy on the load balancer security group
cft.resources.add(Resource('ELBSecurityGroupIngressHTTP',
    'AWS::EC2::SecurityGroupIngress',
    {
        'GroupName': ref('ELBSecurityGroup'),
        'IpProtocol': 'tcp',
        'FromPort': '80',
        'ToPort': '80',
        'SourceSecurityGroupName': get_att('LoadBalancer', 'SourceSecurityGroup.GroupName'),
        'SourceSecurityGroupOwnerId': get_att('LoadBalancer', 'SourceSecurityGroup.OwnerAlias')
    })
)

# Let folks SSH up to the instance
cft.resources.add(Resource('SSHSecurityGroup',
    'AWS::EC2::SecurityGroup',
    {
        'GroupDescription': 'allows inbound SSH from all',
        'SecurityGroupIngress': {
            'IpProtocol': 'tcp',
            'CidrIp': '0.0.0.0/0',
            'FromPort': '22',
            'ToPort': '22'
        }
    })
)

# Make our auto scaling group
cft.resources.add(Resource('AppServerAutoScalingGroup',
    'AWS::AutoScaling::AutoScalingGroup',
    {
        'AvailabilityZones': options['AppServerAvailabilityZones'],
        'HealthCheckGracePeriod': 300,
        'HealthCheckType': 'ELB',
        'LaunchConfigurationName': ref('AppServerAutoScalingLaunchConfig'),
        'LoadBalancerNames': [ref('LoadBalancer')],
        'MaxSize': options['AutoScalingGroupMaxSize'],
        'MinSize': options['AutoScalinggroupMinSize'],
        'Tags': [{
            'Key': 'Name',
            'Value': options['StackRole'] + '-static-app-server',
            'PropagateAtLaunch': True,
        }]
    })
)
# Create the auto scaling group configuration for managing the server instances
cft.resources.add(Resource('AppServerAutoScalingLaunchConfig',
    'AWS::AutoScaling::LaunchConfiguration',
    {
        'ImageId': options['ImageId'],
        'InstanceType': options['AppServerInstanceType'],
        'KeyName': options['KeyPair'],
        'SecurityGroups': [
            ref('ELBSecurityGroup'),
            ref('SSHSecurityGroup'),
        ],
        # Another way to pass a user-data script,
        # looks better than the first example, but it's more tedious
        'UserData': base64(join('\n',
            '#!/bin/bash -v',
            '# do stuff...',
            '# ',
            '# Like maybe kick off cfnbootstrap, using all of the',
            '# AWS:CloudFormation::Init Metadata That we could have',
            '# put on our AutoScalingGroup',
            'exit 0',
        ))
    })
)

# Scale up policy for when the scale up alarm trips
cft.resources.add(Resource('AppServerScaleUpPolicy',
    'AWS::AutoScaling::ScalingPolicy',
    {
        'AdjustmentType': 'ChangeInCapacity',
        'AutoScalingGroupName': ref('AppServerAutoScalingGroup'),
        'Cooldown': '600',
        'ScalingAdjustment': '1'
    })
)

# Scale down policy for when the scale down alarm trips
cft.resources.add(Resource('AppServerScaleDownPolicy',
    'AWS::AutoScaling::ScalingPolicy',
    {
        'AdjustmentType': 'ChangeInCapacity',
        'AutoScalingGroupName': ref('AppServerAutoScalingGroup'),
        'Cooldown': '600',
        'ScalingAdjustment': '-1'
    })
)

# CloudWatch scale up alarm for triggering scale events
cft.resources.add(Resource('AppServerCPUAlarmHigh',
    'AWS::CloudWatch::Alarm',
    {
        'AlarmDescription': 'Scale up if average CPU usage of the AppServers stays above 75% for at least 5 minutes',
        'Dimensions': [{'Name': 'AutoScalingGroupName', 'Value': ref('AppServerAutoScalingGroup')}],
        'Namespace': 'AWS/EC2',
        'MetricName': 'CPUUtilization',
        'Unit': 'Percent',
        'Period': '60',
        'EvaluationPeriods': '5',
        'Statistic': 'Average',
        'ComparisonOperator': 'GreaterThanThreshold',
        'Threshold': '75',
        'ActionsEnabled': options['CloudWatchAlarmActionsEnabled'],
        'AlarmActions': [ref('AppServerScaleUpPolicy')]
    })
)

# CloudWatch scale down alarm for triggering scale events
cft.resources.add(Resource('AppServerCPUAlarmLow',
    'AWS::CloudWatch::Alarm',
    {
        'AlarmDescription': 'Scale down if average CPU usage of the AppServers stays below 25% for at least 5 minutes',
        'Dimensions': [{'Name': 'AutoScalingGroupName', 'Value': ref('AppServerAutoScalingGroup')}],
        'Namespace': 'AWS/EC2',
        'MetricName': 'CPUUtilization',
        'Unit': 'Percent',
        'Period': '60',
        'EvaluationPeriods': '5',
        'Statistic': 'Average',
        'ComparisonOperator': 'LessThanThreshold',
        'Threshold': '25',
        'ActionsEnabled': options['CloudWatchAlarmActionsEnabled'],
        'AlarmActions': [ref('AppServerScaleUpPolicy')]
    })
)

# Add the load balancer endpoint to our outputs
cft.outputs.add(
    Output('LoadBalancerEndpoint',
        get_att('LoadBalancer', 'DNSName')
    )
)



# Copyright (c) 2013 MetaMetrics, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

from collections import defaultdict

prompt_str = '''Key "{0}" not found in the supplied options mapping.
You can enter it now (or leave blank for None/null):
> '''


class OptionsMapping(defaultdict):
    def __init__(self, *args, **kwargs):
        super(OptionsMapping, self).__init__(None, *args, **kwargs)

    def __missing__(self, key):
        try:
            value = raw_input(prompt_str.format(key))
        except KeyboardInterrupt:
            # Catch the sigint here, since the user's pretty likely to
            # Ctrl-C and go fix the options mapping input file
            raise SystemExit

        if not value:
            value = None

        self[key] = value
        return value



cft = CloudFormationTemplate(description='My project template.')

cft.parameters.add(
    Parameter('EC2InstanceType', 'String',
        {
            'Default': 'm1.small',
            'Description': 'Instance type to use for created Server EC2 instance',
            'AllowedPattern': 'm3.[a-z]+',
            'ConstraintDescription': 'Must use one of the m3 instance types.',
        }
    )
)

cft.resources.add(
    Resource('Server', 'AWS::EC2::Instance',
        {
            'ImageId': options['AmiId'],
            'InstanceType': ref('EC2InstanceType')
        }
    )
)

cft.resources.add(
    Resource('StaticFiles', 'AWS::S3::Bucket', {'AccessControl: PublicRead'}, Metadata(
        {'Object1': 'Location1', 'Object2': 'Location2'}
    ))
)



user_data_script = '''#!/bin/bash

echo "You can put your userdata script right here!"
'''

cft = CloudFormationTemplate(description="A slightly more useful template.")
properties = {
    'ImageId': 'ami-c30360aa',
    'InstanceType': 'm1.small',
    'UserData': base64(user_data_script),
}
attributes = [
    Metadata(
        {
            "AWS::CloudFormation::Init": {
                "config": {
                    "packages": {},
                    "sources": {},
                    "commands": {},
                    "files": {},
                    "services": {},
                    "users": {},
                    "groups": {}
                }
            }
        }
    ),
]

cft.resources.add(
    Resource('MyInstance', 'AWS::EC2::Instance', properties, attributes)
)



cft = CloudFormationTemplate(description="A slightly more useful template.")

user_data_script = '''#!/bin/bash

echo "You can put your userdata script right here!"
'''

cft.resources.add(Resource('AnInstance', 'AWS::EC2::Instance',
    {
        'ImageId': 'ami-c30360aa',
        'InstanceType': 'm1.small',
        'UserData': base64(user_data_script),
    })
)



user_data_script = '''#!/bin/bash

echo "You can put your userdata script right here!"
'''

cft = CloudFormationTemplate(description="A slightly more useful template.")
properties = {
    'ImageId': 'ami-c30360aa',
    'InstanceType': 'm1.small',
    'UserData': base64(user_data_script),
}
attributes = [
    Metadata(
        {
            "AWS::CloudFormation::Init": {
                "config": {
                    "packages": {
                        "rpm": {
                            "epel": "http://download.fedoraproject.org/pub/epel/5/i386/epel-release-5-4.noarch.rpm"
                        },
                        "yum": {
                            "httpd": [],
                            "php": [],
                            "wordpress": []
                        },
                        "rubygems": {
                            "chef": ["0.10.2"]
                        }
                    },
                    "sources": {
                        "/etc/puppet": "https://github.com/user1/cfn-demo/tarball/master"
                    },
                    "commands": {
                        "test": {
                            "command": "echo \"$CFNTEST\" > test.txt",
                            "env": {"CFNTEST": "I come from config1."},
                            "cwd": "~",
                            "test": "test ! -e ~/test.txt",
                            "ignoreErrors": "false"
                        }
                    },
                    "files": {
                        "/tmp/setup.mysql": {
                            "content":
                                join('',
                                     "CREATE DATABASE ", ref("DBName"), ";\n",
                                     "CREATE USER '", ref("DBUsername"), "'@'localhost' IDENTIFIED BY '",
                                     ref("DBPassword"),
                                     "';\n",
                                     "GRANT ALL ON ", ref("DBName"), ".* TO '", ref("DBUsername"), "'@'localhost';\n",
                                     "FLUSH PRIVILEGES;\n"
                                ),
                            "mode": "000644",
                            "owner": "root",
                            "group": "root"
                        }
                    },
                    "services": {
                        "sysvinit": {
                            "nginx": {
                                "enabled": "true",
                                "ensureRunning": "true",
                                "files": ["/etc/nginx/nginx.conf"],
                                "sources": ["/var/www/html"]
                            },
                            "php-fastcgi": {
                                "enabled": "true",
                                "ensureRunning": "true",
                                "packages": {
                                    "yum": ["php", "spawn-fcgi"]
                                }
                            },
                            "sendmail": {
                                "enabled": "false",
                                "ensureRunning": "false"
                            }
                        }
                    },
                    "users": {
                        "myUser": {
                            "groups": ["groupOne", "groupTwo"],
                            "uid": "50",
                            "homeDir": "/tmp"
                        }
                    },
                    "groups": {
                        "groupOne": {
                        },
                        "groupTwo": {
                            "gid": "45"
                        }
                    }
                }
            }
        }
    ),
    UpdatePolicy(
        {
            "AutoScalingRollingUpdate": {
                "MinInstancesInService": "1",
                "MaxBatchSize": "1",
                "PauseTime": "PT12M5S"
            }
        }
    ),
    DeletionPolicy("Retain"),
    DependsOn(ref("myDB"))
]
cft.resources.add(
    Resource('MyInstance', 'AWS::EC2::Instance', properties, attributes)
)
